
//-----------------------------------------------------------------------------
// File: grbrdemo.cpp
//
// Desc: Grabber demo code based on the Microsoft Playcap Sample with support
//		 for the Video Mixing Renderer
//
// 17/01/13 Advanced Micro Peripherals Ltd.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// INCLUDE FILES
//-----------------------------------------------------------------------------

#include <atlbase.h>
#include <windows.h>
#include <dshow.h>
#include <stdio.h>
#include <mtype.h>	// for DeleteMediaType
#include <math.h>

#include "grbrdemo.h"

//-----------------------------------------------------------------------------
// PREPROCESSOR DEFINITIONS
//-----------------------------------------------------------------------------

// An application can advertise the existence of its filter graph	
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//
#define REGISTER_FILTERGRAPH

// Bitmap header information

#pragma pack(1)
typedef struct {
    char    bfType[2];
    long    filesize;
    short   reserved1;
    short   reserved2;
    long    headersize;
    long    biSize;
    long    biWidth;
    long    biHeight;
    short   biPlanes;
    short   biBitCount;
    long    biCompression;
    long    biSizeImage;
    long    biXPelsPerMeter;
    long    biYPelsPerMeter;
    long    biClrUsed;
    long    biClrImportant;
    } BMPHEAD;
#pragma pack()

/* constants for the biCompression field */
#define BI_RGB      0L
#define BI_RLE8     1L
#define BI_RLE4     2L

//-----------------------------------------------------------------------------
// GLOBAL DATA
//-----------------------------------------------------------------------------

HWND ghApp=0;
HWND ghFullScreen=0;
HWND ghSurface=0;
DWORD g_dwGraphRegister=0;
HMENU ghMenu=0;
HINSTANCE ghInst=0;

IMediaControl				*g_pMC = NULL;
IMediaEventEx				*g_pME = NULL;
IGraphBuilder				*g_pGraph = NULL;
ICaptureGraphBuilder2		*g_pCapture = NULL;
IBaseFilter					*g_pGRABBER[8] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
IBaseFilter					*g_pBF = NULL;
IAMStreamConfig				*g_pVSCC[8] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
IAMAnalogVideoDecoder		*g_pAnalogVideoDecoder[8] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
IVMRWindowlessControl9		*g_Wc;
IVMRMixerControl9			*g_pMixControl;

PLAYSTATE g_psCurrent = Stopped;

WCHAR wachTxtFriendlyName[120];

BOOL g_bFullscreen = FALSE;
BOOL g_bGraphBuilt = FALSE;
int g_Index = 1;

typedef struct tagCapSize{
	int width;
	int height;
} CAPTURESIZE;

CAPTURESIZE capsize[] = {
	{ 160, 120 },
	{ 176, 144 },
	{ 240, 176 },
	{ 240, 180 },
	{ 320, 240 },
	{ 352, 240 },
	{ 352, 288 },
	{ 640, 480 },
	{ 720, 480 },
	{ 704, 576 },
	{ 720, 576 }
};
int capsize_idx = 7;	// default to 640x480

#define NTSC_MAX_IDX	8

int g_VidWidth = capsize[capsize_idx].width;
int g_VidHeight = capsize[capsize_idx].height;

BOOL g_bPal = TRUE;

//-----------------------------------------------------------------------------
// FUNCTION PROTOTYPES
//-----------------------------------------------------------------------------

void StartGraph(int index);
void NukeDownstream(IBaseFilter *pf);

//-----------------------------------------------------------------------------
// FUNCTION DEFINITIONS
//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Function name	: GetInterfaces
// Description	    : This function gets the basic interfaces that are used 
//					  throught the APP.
// Return type		: HRESULT 
// Argument         : void
// Returns          : 
//
HRESULT GetInterfaces(void)
{
    HRESULT hr;

    // Create the filter graph
    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC,
        IID_IGraphBuilder, (void **) &g_pGraph);
    if (FAILED(hr))
        return hr;

    // Create the capture graph builder
    hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
        IID_ICaptureGraphBuilder2, (void **) &g_pCapture);
    if (FAILED(hr))
        return hr;
    
    // Obtain interfaces for media control and Video Window
    hr = g_pGraph->QueryInterface(IID_IMediaControl,(LPVOID *) &g_pMC);
    if (FAILED(hr))
        return hr;

    hr = g_pGraph->QueryInterface(IID_IMediaEvent, (LPVOID *) &g_pME);
    if (FAILED(hr))
        return hr;

    // Set the window handle used to process graph events
    hr = g_pME->SetNotifyWindow((OAHWND)ghApp, WM_GRAPHNOTIFY, 0);

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: CloseInterfaces
// Description	    : Closes all of the interfaces allocated by GetInterfaces().
// Return type		: void 
// Argument         : void
// Returns          : 
//
void CloseInterfaces(void)
{
	int i;
    // Stop previewing data
    if (g_pMC)
        g_pMC->StopWhenReady();

    g_psCurrent = Stopped;

    // Stop receiving events
    if (g_pME)
        g_pME->SetNotifyWindow(NULL, WM_GRAPHNOTIFY, 0);

    // Relinquish ownership (IMPORTANT!) of the video window.
    // Failing to call put_Owner can lead to assert failures within
    // the video renderer, as it still assumes that it has a valid
    // parent window.

#ifdef REGISTER_FILTERGRAPH
    // Remove filter graph from the running object table   
    if (g_dwGraphRegister)
        RemoveGraphFromRot(g_dwGraphRegister);
#endif

    // Release DirectShow interfaces
	for(i=0;i<8;i++)
	{
		SAFE_RELEASE(g_pAnalogVideoDecoder[i]);
		SAFE_RELEASE(g_pVSCC[i]);
	}
    SAFE_RELEASE(g_pMC);
    SAFE_RELEASE(g_pME);
    SAFE_RELEASE(g_pGraph);
    SAFE_RELEASE(g_pCapture);
	SAFE_RELEASE(g_pBF);
}

//------------------------------------------------------------------------------
// Function name	: FindCaptureDevice
// Description	    : Finds the first Video Capture Device in the system
// Return type		: HRESULT 
// Argument         : IBaseFilter ** ppSrcFilter
// Returns          : 
//
HRESULT FindCaptureDevice(IBaseFilter ** ppSrcFilter, int index)
{
    HRESULT hr;
    IBaseFilter * pSrc = NULL;
    CComPtr <IMoniker> pMoniker =NULL;
    ULONG cFetched;
   
    // Create the system device enumerator
    CComPtr <ICreateDevEnum> pDevEnum =NULL;

    hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
        IID_ICreateDevEnum, (void ** ) &pDevEnum);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create system enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // Create an enumerator for the video capture devices
    CComPtr <IEnumMoniker> pClassEnum = NULL;

    hr = pDevEnum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create class enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // If there are no enumerators for the requested type, then 
    // CreateClassEnumerator will succeed, but pClassEnum will be NULL.
    if (pClassEnum == NULL)
    {
        MessageBox(ghApp,TEXT("No video capture device was detected.\r\n\r\n")
                   TEXT("This sample requires a video capture device, such as a USB WebCam,\r\n")
                   TEXT("to be installed and working properly.  The sample will now close."),
                   TEXT("No Video Capture Hardware"), MB_OK | MB_ICONINFORMATION);
        return E_FAIL;
    }

    // Use the index video capture device on the device list.
	if(index)
		pClassEnum->Skip(index);

    // Note that if the Next() call succeeds but there are no monikers,
    // it will return S_FALSE (which is not a failure).  Therefore, we
    // check that the return code is S_OK instead of using SUCCEEDED() macro.
    if (S_OK == (pClassEnum->Next (1, &pMoniker, &cFetched)))
    {
        // Bind Moniker to a filter object
        hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**)&pSrc);
        if (FAILED(hr))
        {
            Msg(TEXT("Couldn't bind moniker to filter object!  hr=0x%x"), hr);
            return hr;
        }
    }
    else
    {
        Msg(TEXT("Unable to access video capture device!"));   
        return E_FAIL;
    }

    // Copy the found filter pointer to the output parameter.
    // Do NOT Release() the reference, since it will still be used
    // by the calling function.
    *ppSrcFilter = pSrc;

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: BuildGraph
// Description	    : Builds the direct show filter graph that this app uses.
//					  Relies on some globals to make decisions when building
//					  graph.
// Return type		: HRESULT 
// Returns          :
//
HRESULT BuildGraph(int index)
{
    HRESULT hr;

    // Get DirectShow interfaces
    hr = GetInterfaces();
    if (FAILED(hr))
    {
        Msg("Failed to get video interfaces!  hr=0x%x", hr);
        return hr;
    }

    // Attach the filter graph to the capture graph
    hr = g_pCapture->SetFiltergraph(g_pGraph);
    if (FAILED(hr))
    {
        Msg("Failed to set capture filter graph!  hr=0x%x", hr);
        return hr;
    }
    IBaseFilter* pBF = NULL;
    hr = CoCreateInstance(CLSID_VideoMixingRenderer9, NULL, CLSCTX_INPROC,
                                    IID_IBaseFilter, (LPVOID *)&pBF);
    if (FAILED(hr))
    {
        Msg("Failed to create VMR filter!  hr=0x%x", hr);
        return hr;
    }	

    hr = g_pGraph->AddFilter(pBF, L"Video Mixing Renderer 9");
	if (FAILED(hr))
    {
        Msg("Failed to add VMR filter!  hr=0x%x", hr);
        return hr;
    }	

	IVMRFilterConfig9* pConfig;
	HRESULT hRes2 = pBF->QueryInterface(IID_IVMRFilterConfig9, (LPVOID *)&pConfig);
	if(SUCCEEDED(hRes2))
	{
		hRes2 = pConfig->SetNumberOfStreams(8);
		hRes2 = pConfig->SetRenderingMode(VMR9Mode_Windowless);
		//hRes2 = pConfig->SetRenderingPrefs(RenderPrefs_AllowOverlays);
		pConfig->Release();
	}
	
	IVMRMonitorConfig9* pMonitorConfig;
	HRESULT hRes3 = pBF->QueryInterface(IID_IVMRMonitorConfig9, (LPVOID *)&pMonitorConfig);
	if(SUCCEEDED(hRes3))
	{
		UINT iCurrentMonitor;
		HRESULT hr4 = pMonitorConfig->GetMonitor(&iCurrentMonitor);
		pMonitorConfig->Release();
	}

	hr = pBF->QueryInterface(IID_IVMRWindowlessControl9, (LPVOID *)&g_Wc);

	hr = g_Wc->SetVideoClippingWindow(ghSurface);
	
	hr = g_Wc->SetAspectRatioMode(VMR_ARMODE_NONE);

	hr = g_Wc->QueryInterface(IID_IVMRMixerControl9, (LPVOID *) &g_pMixControl);
    if(pBF)
    {
        pBF->Release();
    }
    // Use the system device enumerator and class enumerator to find
    // a video capture/preview device.

	int i;
	for(i=0;i<index;i++)
	{
		VMR9NormalizedRect r;
			IEnumPins *pEnum = NULL;
			IFilterGraph2 *pGraph2 = NULL;
			IPin *pPin = NULL;
		hr = FindCaptureDevice(&g_pGRABBER[i], i);
		if (FAILED(hr))
		{
			Msg("Device %d/%d failed",i,index);
			// Don't display a message because FindCaptureDevice will handle it
			break;
		}	
   
	    // Add Capture filter to our graph.
		hr = g_pGraph->AddFilter(g_pGRABBER[i], L"Video Capture");
		if (FAILED(hr))
		{
	        Msg("Couldn't add capture filter to graph!  hr=0x%x", hr);
			SAFE_RELEASE(g_pGRABBER[i]);
	        break;
		}

		// Get pointers to useful interfaces
		hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, g_pGRABBER[i], IID_IAMStreamConfig, (void **)&g_pVSCC[i]);

		if(FAILED(hr))
		{
			Msg("Couldn't get IAMStreamConfig Interface (Capture)\n");
			SAFE_RELEASE(g_pGRABBER[i]);
			return hr;
		}
	
		hr = g_pGRABBER[i]->QueryInterface(IID_IAMAnalogVideoDecoder, (void **)&g_pAnalogVideoDecoder[i]);
		if(FAILED(hr))
		{
			Msg("Couldn't get IAMAnalogVideoDecoder\n");
			SAFE_RELEASE(g_pGRABBER[i]);
			return hr;
		}
	
		hr = g_pGraph->QueryInterface(IID_IFilterGraph2, (void**)&pGraph2);
		hr = g_pGRABBER[i]->EnumPins(&pEnum);
		bool bRenderedAnyPin=FALSE;
		HRESULT hr2;
		while (S_OK == pEnum->Next(1, &pPin, NULL))
			{			
				// Try to render this pin. 
				// It's OK if we fail some pins, if at least one pin renders.
				hr2 = pGraph2->RenderEx(pPin, AM_RENDEREX_RENDERTOEXISTINGRENDERERS, NULL);

				pPin->Release();

				if (SUCCEEDED(hr2))
				{
					bRenderedAnyPin = TRUE;
				}
			}
		if(bRenderedAnyPin==FALSE)
			Msg("Pin failed: %x\n",hr2);

		// Calculate the channel positioning based on the number of channels.
		// Use the square root to aim for an n*n matrix (or the closest possible)
		// Round up so we have space for more videos than we have, rather than not enough.
		float nX=ceil(sqrt((float)index));
		float nY=ceil(index/nX);

		// Co-ordinates are represented as fractions of the whole surface.
		r.left=(float)(i%(int)nX)/nX;
		r.top=(float)(i/(int)nX)/nY;
		r.right=1/nX+(float)(i%(int)nX)/nX;
		r.bottom=1/nY+(float)(i/(int)nX)/nY;

		g_pMixControl->SetOutputRect(i, &r);

		// Set alpha to 100% as we don't want to blend.
		g_pMixControl->SetAlpha(i, 1);

		// default to PAL.
		if(g_bPal)
			g_pAnalogVideoDecoder[i]->put_TVFormat(AnalogVideo_PAL_I);
		else
			g_pAnalogVideoDecoder[i]->put_TVFormat(AnalogVideo_NTSC_M);
	}

	g_bGraphBuilt = TRUE;

	return S_OK;
}

HRESULT DestroyGraph()
{
	HRESULT hr;
	if(g_bGraphBuilt)
	{
		int i;
		for(i=0;i<8;i++)
			if(g_pGRABBER[i])
			{
				NukeDownstream(g_pGRABBER[i]);
				hr=g_pGraph->RemoveFilter(g_pGRABBER[i]);
				SAFE_RELEASE(g_pGRABBER[i]);
			}
		hr=g_pGraph->RemoveFilter(g_pBF);
		SAFE_RELEASE(g_pBF);

		CloseInterfaces();		

		g_bGraphBuilt = FALSE;
	}

	return S_OK;
}

HRESULT SetPreviewSize(int channel, int width, int height)
{
	HRESULT hr;

	// Set the video size on the capture pin to our desired size and format.
	if(g_pVSCC[channel])
	{
		AM_MEDIA_TYPE *pmt;
		hr = g_pVSCC[channel]->GetFormat(&pmt);
		if(FAILED(hr))
		{
			Msg("Couldn't read current media format\n");
			return hr;
		}
		else
		{
			// need to check that we have the right format type.
			if(pmt->formattype == FORMAT_VideoInfo)
			{
				long biWidth, biHeight;
				DWORD biSizeImage;
				VIDEOINFOHEADER *pvih = (VIDEOINFOHEADER *)pmt->pbFormat;
				biWidth = width;
				biHeight = height;
				biSizeImage = width * height * 2;

				pvih->bmiHeader.biWidth = biWidth;
				pvih->bmiHeader.biHeight = biHeight;
				pvih->bmiHeader.biSizeImage = biSizeImage;

				hr = g_pVSCC[channel]->SetFormat(pmt);
				
			}
			DeleteMediaType(pmt);
		}
	}
	
	g_bGraphBuilt = TRUE;

    return S_OK;
}

//------------------------------------------------------------------------------
// Function name	: NukeDownstream
// Description	    : Tear down everything downstream of a given filter
// Return type		: void 
// Argument         : IBaseFilter *pf
// Returns          : 
//
void NukeDownstream(IBaseFilter *pf) 
{
	IPin *pP, *pTo;
    ULONG u;
    IEnumPins *pins = NULL;
    PIN_INFO pininfo;
    HRESULT hr = pf->EnumPins(&pins);
    pins->Reset();

    while(hr == NOERROR) 
	{
        hr = pins->Next(1, &pP, &u);
        if(hr == S_OK && pP) 
		{
            hr=pP->ConnectedTo(&pTo);
            if(pTo) 
			{
                hr = pTo->QueryPinInfo(&pininfo);
                if(hr == NOERROR) 
				{
                    if(pininfo.dir == PINDIR_INPUT) 
					{
						NukeDownstream(pininfo.pFilter);
                        hr=g_pGraph->Disconnect(pTo);
						g_pGraph->Disconnect(pP);
						g_pGraph->RemoveFilter(pininfo.pFilter);
                    }
                    pininfo.pFilter->Release();
                }
                pTo->Release();
            }
            pP->Release();
        }
    }
    if(pins)
        pins->Release();
}

//------------------------------------------------------------------------------
// Function name	: NukeDownstream
// Description	    : Tear down everything downstream of a given filter
// Return type		: void 
// Argument         : IBaseFilter *pf
// Returns          : 
//
void DisconnectDownstream(IBaseFilter *pf) 
{
    IPin *pP, *pTo;
    ULONG u;
    IEnumPins *pins = NULL;
    PIN_INFO pininfo;
    HRESULT hr = pf->EnumPins(&pins);
    pins->Reset();

    while(hr == NOERROR) 
	{
        hr = pins->Next(1, &pP, &u);
        if(hr == S_OK && pP) 
		{
            pP->ConnectedTo(&pTo);
            if(pTo) 
			{
                hr = pTo->QueryPinInfo(&pininfo);
                if(hr == NOERROR) 
				{
                    if(pininfo.dir == PINDIR_INPUT) 
					{
                        DisconnectDownstream(pininfo.pFilter);
                        g_pGraph->Disconnect(pTo);
                        g_pGraph->Disconnect(pP);
                    }
                }
                pTo->Release();
            }
            pP->Release();
        }
    }
    if(pins)
        pins->Release();
}

//------------------------------------------------------------------------------
// Function name	: Capture
// Description	    : Start the graph.
// Return type		: HRESULT 
// Returns          : 
//
HRESULT Capture()
{
    HRESULT hr;

    // Set video window style and position
    hr = SetupVideoWindow();
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't initialize video window!  hr=0x%x"), hr);
        return hr;
    }

	// Add our graph to the running object table, which will allow
    // the GraphEdit application to "spy" on our graph
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(g_pGraph, &g_dwGraphRegister);
    if (FAILED(hr))
    {
        Msg(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr);
        g_dwGraphRegister = 0;
    }
#endif

    // Start previewing video data
    hr = g_pMC->Run();
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't run the graph!  hr=0x%x"), hr);
        return hr;
    }

    // Remember current state
    g_psCurrent = Running;
        
    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
/// SetupVideoWindow
/// 
/// Sets up the application window so that ic can contain our capture.
/// 
/// \param
///     void
/// 
/// \return
///     HRESULT 
///
/// \remark
///
///
HRESULT SetupVideoWindow(void)
{
    HRESULT hr = S_OK;
	if(g_Wc)
	{
        RECT src={0}, dest={0};
		GetClientRect(ghSurface, &dest);
		g_Wc->SetVideoPosition(NULL,&dest);
		return hr;
	}
    return hr;
}

//------------------------------------------------------------------------------
// Function name	: ResizeVideoWindow
// Description	    : Correctly sizes our window.
// Return type		: void 
// Argument         : void
// Returns          : 
//
void ResizeVideoWindow(void)
{
    RECT rc;

    // Make the preview video fill our window
    GetClientRect(ghApp, &rc);

    // Resize the video preview window to match owner window size
	
    if(g_Wc)
	{
        HDC hdcWin = GetDC(NULL);
        g_Wc->SetVideoPosition(NULL, &rc);
		g_Wc->RepaintVideo(ghSurface, hdcWin);
	}
    
}

//------------------------------------------------------------------------------
// Function name	: ChangePreviewState
// Description	    : Change the current running state of the graph
// Return type		: HRESULT 
// Argument         : int nShow
// Returns          : 
//
HRESULT ChangePreviewState(int nShow)
{
    HRESULT hr=S_OK;
    
    // If the media control interface isn't ready, don't call it
    if (!g_pMC)
        return S_OK;
    
    if (nShow)
    {
        if (g_psCurrent != Running)
        {
            // Start previewing video data
            hr = g_pMC->Run();
            g_psCurrent = Running;
        }
    }
    else
    {
        // Stop previewing video data
        hr = g_pMC->StopWhenReady();
        g_psCurrent = Stopped;
    }

    return hr;
}

#ifdef REGISTER_FILTERGRAPH

//------------------------------------------------------------------------------
// Function name	: AddGraphToRot
// Description	    : Utility function. Allows graph to be connected to using
//					  graphEdit from MS.
// Return type		: HRESULT 
// Argument         : IUnknown *pUnkGraph
// Argument         : DWORD *pdwRegister
// Returns          : 
//
HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
    IMoniker * pMoniker;
    IRunningObjectTable *pROT;
    WCHAR wsz[128];
    HRESULT hr;

    if (FAILED(GetRunningObjectTable(0, &pROT))) 
	{
        return E_FAIL;
    }

    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());

    hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) 
	{
        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();
    return hr;
}

//------------------------------------------------------------------------------
// Function name	: RemoveGraphFromRot
// Description	    : Utility function. Allows graph to be connected to using
//					  graphEdit from MS.
// Return type		: void 
// Argument         : DWORD pdwRegister
// Returns          : 
//
void RemoveGraphFromRot(DWORD pdwRegister)
{
    IRunningObjectTable *pROT;

    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) 
	{
        pROT->Revoke(pdwRegister);
        pROT->Release();
    }
}

#endif

//------------------------------------------------------------------------------
// Function name	: Msg
// Description	    : Simple message dump function
// Return type		: void 
// Argument         : TCHAR *szFormat
// Argument         : ...
// Returns          : 
//
void Msg(TCHAR *szFormat, ...)
{
    TCHAR szBuffer[512];

    va_list pArgs;
    va_start(pArgs, szFormat);
    _vstprintf_s(szBuffer, 512, szFormat, pArgs);
    va_end(pArgs);

    MessageBox(NULL, szBuffer, TEXT("PlayCap Message"), MB_OK | MB_ICONERROR);
}

//------------------------------------------------------------------------------
// Function name	: ToggleFullScreen
// Description	    : Toggles fullscreen display of the captured video
// Return type		: HRESULT 
// Argument         : void
// Returns          :
//
HRESULT ToggleFullScreen(void)
{
    HRESULT hr=S_OK;
    static HWND hDrain=0;
	RECT rc;


    if (g_bFullscreen == FALSE)
    {
		ShowWindow(ghFullScreen, SW_SHOWMAXIMIZED);
		ShowWindow(ghApp, SW_HIDE);
		g_Wc->SetVideoClippingWindow(ghFullScreen);
        g_bFullscreen = TRUE;
		ghSurface=ghFullScreen;

		GetClientRect(ghFullScreen, &rc);
	    g_Wc->SetVideoPosition(NULL, &rc);
    }
    else
    {
        // Switch back to windowed mode
		ShowWindow(ghFullScreen, SW_HIDE);
		ShowWindow(ghApp, SW_SHOWNORMAL);
		g_Wc->SetVideoClippingWindow(ghApp);
        g_bFullscreen = FALSE;
		ghSurface=ghApp;
		GetClientRect(ghApp, &rc);
	    g_Wc->SetVideoPosition(NULL, &rc);
    }

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: HandleGraphEvent
// Description	    : 
// Return type		: HRESULT 
// Argument         : void
// Returns          : 
//
HRESULT HandleGraphEvent(void)
{
    LONG evCode, evParam1, evParam2;
    HRESULT hr=S_OK;

    while(SUCCEEDED(g_pME->GetEvent(&evCode, (LONG_PTR *) &evParam1, (LONG_PTR *) &evParam2, 0)))
    {
        // Free event parameters to prevent memory leaks associated with
        // event parameter data.  While this application is not interested
        // in the received events, applications should always process them.
        hr = g_pME->FreeEventParams(evCode, evParam1, evParam2);
        
        // Insert event processing code here, if desired
    }

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: WndMainProc 
// Description	    : Main Window Proc
// Return type		: LRESULT CALLBACK 
// Argument         : HWND hwnd
// Argument         : UINT message
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Returns          : 
//
LRESULT CALLBACK WndMainProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int index;
	long lDecoder=0;

    switch (message)
    {
        case WM_GRAPHNOTIFY:
            HandleGraphEvent();
            break;

        case WM_KEYDOWN:

            switch(toupper((int) wParam))
            {
				// This toggles the input standard between PAL and NTSC.
				case 'I':
					if(g_bPal)
					{
						g_bPal = FALSE;
					}
					else
					{
						g_bPal = TRUE;
					}
					g_pMC->Stop();
					DestroyGraph();
					StartGraph(g_Index);
					break;

				case 'F':
                case VK_RETURN:
				case VK_SPACE:
                    ToggleFullScreen();
                    break;

				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
					index = wParam-'0';
					if(index!=g_Index)
					{
						g_pMC->Stop();
						g_Index=index;
						DestroyGraph();
						StartGraph(g_Index);
					}
					break;

				case 'S':
					g_pMC->Stop();
					break;

				case 'R':
					g_pMC->Run();
					break;

                case VK_ESCAPE:
                    if(g_bFullscreen)
                        ToggleFullScreen();
                    break;

				// Quits the application
				case 'Q':
		            PostMessage(hwnd,WM_CLOSE,0,0L);
					break;
			break;
            }
            break;

        case WM_CLOSE:            
            // Hide the main window while the graph is destroyed
            ShowWindow(ghApp, SW_HIDE);
            CloseInterfaces();  // Stop capturing and release interfaces
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            return 0;

		case WM_POWERBROADCAST:
			switch(wParam)
			{
                case PBT_APMQUERYSUSPEND:
                    // At this point we should prepare to go into a suspended mode. if we cannot return BROADCAST_QUERY_DENY
                    return TRUE;

				case PBT_APMSUSPEND:
					// No choice, we are suspending do what we can to get ourselves to the right state.
					break;

				case PBT_APMRESUMESUSPEND:
					// At this point we should restore our settings following the suspend.
					return TRUE;

				default:
					break;
			}
			break;

		case WM_MOVE:
		case WM_SIZE:
			ResizeVideoWindow();
			break;
    }
    return DefWindowProc (hwnd , message, wParam, lParam);
}

// ----------------------------------------------------------------------------
// MAIN FUNCTION
// ----------------------------------------------------------------------------

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hInstP, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;

    WNDCLASS wc;

    // Initialize COM
    if(FAILED(CoInitialize(NULL)))
    {
        Msg(TEXT("CoInitialize Failed!\r\n"));   
        exit(1);
    } 

    // Register the window class
    ZeroMemory(&wc, sizeof wc);
    wc.lpfnWndProc   = WndMainProc;
    ghInst = wc.hInstance = hInstance;
    wc.lpszClassName = CLASSNAME;
    wc.lpszMenuName  = NULL;
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VIDPREVIEW));
    if(!RegisterClass(&wc))
    {
        Msg(TEXT("RegisterClass Failed! Error=0x%x\r\n"), GetLastError());
        CoUninitialize();
        exit(1);
    }

    // Create the main window.  The WS_CLIPCHILDREN style is required.
    ghApp = CreateWindow(CLASSNAME, 
					APPLICATIONNAME,
                    WS_OVERLAPPEDWINDOW | WS_CAPTION | WS_CLIPCHILDREN,
                    CW_USEDEFAULT, 
					CW_USEDEFAULT,
                    g_VidWidth, 
					g_VidHeight,
                    0, 
					0, 
					hInstance, 
					0);

	ghSurface=ghApp;
	
	ghFullScreen=CreateWindow(CLASSNAME, 
					"",
                    WS_DISABLED |WS_POPUP,
					0,
					0,
                    CW_USEDEFAULT, 
					CW_USEDEFAULT,
                    0, 
					0, 
					hInstance, 
					0);

    if(ghApp)
    {
		// Save menu handle for later use
		ghMenu = GetMenu(ghApp);

		ShowWindow(ghApp, nCmdShow);

		StartGraph(g_Index);

        // Main message loop
        while(GetMessage(&msg,NULL,0,0))
        {
			TranslateMessage(&msg) ;
			DispatchMessage(&msg) ;
        }
    }
    // Release COM
    CoUninitialize();

    return (int) msg.wParam;
}

void StartGraph(int index)
{
	HRESULT hr;

	if(!g_bGraphBuilt)
	{
		// Create DirectShow graph and start capturing video
		hr = BuildGraph(index);
		if (FAILED (hr))
		{
			CloseInterfaces();
			DestroyWindow(ghApp);
		}
		else
		{
			int i;
			for(i=0;i<index;i++)
			{
				hr = SetPreviewSize(i, capsize[capsize_idx].width, capsize[capsize_idx].height);
				if(FAILED(hr))
				{
					CloseInterfaces();
					DestroyWindow(ghApp);
					return;
				}
			}
			hr = Capture();
			if (FAILED (hr))
			{
				CloseInterfaces();
				DestroyWindow(ghApp);
				return;
			}
			
			UpdateWindow(ghSurface);
			SetForegroundWindow(ghSurface);
			SetFocus(ghSurface);
			g_bFullscreen = FALSE;
		}
	}
	else
		Msg("Graph Already Running");
}


