//-----------------------------------------------------------------------------
// File: grbrsize.cpp
//
// Desc: Grabber demo capture size code based on the Microsoft Playcap Sample.
//
// 12/08/15 Advanced Micro Peripherals Ltd.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// File: playcap.cpp
//
// Desc: DirectShow sample code - a very basic application using Capture
//       Devices.  It creates a window and uses the first available Capture
//       Device to render and preview video capture data.
//
// Copyright (c) 1999-2001 Microsoft Corporation.  All rights reserved.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// INCLUDE FILES
//-----------------------------------------------------------------------------

#include <atlbase.h>
#include <windows.h>
#include <dshow.h>
#include <stdio.h>
#include <mtype.h>	// for DeleteMediaType

#include "grbrsize.h"

//-----------------------------------------------------------------------------
// PREPROCESSOR DEFINITIONS
//-----------------------------------------------------------------------------

// An application can advertise the existence of its filter graph
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//
#define REGISTER_FILTERGRAPH

//-----------------------------------------------------------------------------
// GLOBAL DATA
//-----------------------------------------------------------------------------

HWND ghApp=0;
DWORD g_dwGraphRegister=0;
HMENU ghMenu=0;
HINSTANCE ghInst=0;

IVideoWindow	*g_pVW = NULL;
IMediaControl	*g_pMC = NULL;
IMediaEventEx	*g_pME = NULL;
IGraphBuilder	*g_pGraph = NULL;
ICaptureGraphBuilder2 *g_pCapture = NULL;
IBaseFilter *g_pGRABBER = NULL;
IAMStreamConfig *g_pVSCC = NULL;
IAMCrossbar *g_pCrossbar = NULL;
IAMAnalogVideoDecoder *g_pAnalogVideoDecoder = NULL;

PLAYSTATE g_psCurrent = Stopped;

WCHAR wachTxtFriendlyName[120];

BOOL g_bFullscreen = FALSE;
BOOL g_bGraphBuilt = FALSE;
int g_Index = 0;

typedef struct tagCapSize{
	int width;
	int height;
} CAPTURESIZE;

#define CAPTURE_SIZE_ENTRIES	8
CAPTURESIZE capsize[CAPTURE_SIZE_ENTRIES][2] = {
	{
		{ 120, 80 },
		{ 120, 96 },
	},
	{
		{ 176, 120 },
		{ 176, 144 },
	},
	{
		{ 320, 240 },
		{ 320, 240 },
	},
	{
		{ 352, 240 },
		{ 352, 288 },
	},
	{
		{ 704, 240 },
		{ 704, 288 },
	},
	{
		{ 640, 480 },
		{ 640, 480 },
	},
	{
		{ 704, 480 },
		{ 704, 576 },
	},
	{
		{ 720, 480 },
		{ 720, 576 }
	}
};

int capsize_idx = 5;	// default to 640x480

BOOL g_bPal = TRUE;

int g_VidWidth = capsize[capsize_idx][g_bPal].width;
int g_VidHeight = capsize[capsize_idx][g_bPal].height;


//-----------------------------------------------------------------------------
// FUNCTION PROTOTYPES
//-----------------------------------------------------------------------------

void StartGraph(int index);
void NukeDownstream(IBaseFilter *pf);

//-----------------------------------------------------------------------------
// FUNCTION DEFINITIONS
//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Function name	: GetInterfaces
// Description	    : This function gets the basic interfaces that are used
//					  throught the APP.
// Return type		: HRESULT
// Argument         : void
// Returns          :
//
HRESULT GetInterfaces(void)
{
    HRESULT hr;

    // Create the filter graph
    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC,
        IID_IGraphBuilder, (void **) &g_pGraph);
    if (FAILED(hr))
        return hr;

    // Create the capture graph builder
    hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
        IID_ICaptureGraphBuilder2, (void **) &g_pCapture);
    if (FAILED(hr))
        return hr;

    // Obtain interfaces for media control and Video Window
    hr = g_pGraph->QueryInterface(IID_IMediaControl,(LPVOID *) &g_pMC);
    if (FAILED(hr))
        return hr;

	hr = g_pGraph->QueryInterface(IID_IVideoWindow, (LPVOID *) &g_pVW);
	if (FAILED(hr))
		return hr;

    hr = g_pGraph->QueryInterface(IID_IMediaEvent, (LPVOID *) &g_pME);
    if (FAILED(hr))
        return hr;

    // Set the window handle used to process graph events
    hr = g_pME->SetNotifyWindow((OAHWND)ghApp, WM_GRAPHNOTIFY, 0);

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: CloseInterfaces
// Description	    : Closes all of the interfaces allocated by GetInterfaces().
// Return type		: void
// Argument         : void
// Returns          :
//
void CloseInterfaces(void)
{
    // Stop previewing data
    if (g_pMC)
        g_pMC->StopWhenReady();

    g_psCurrent = Stopped;

    // Stop receiving events
    if (g_pME)
        g_pME->SetNotifyWindow(NULL, WM_GRAPHNOTIFY, 0);

    // Relinquish ownership (IMPORTANT!) of the video window.
    // Failing to call put_Owner can lead to assert failures within
    // the video renderer, as it still assumes that it has a valid
    // parent window.
    if(g_pVW)
    {
        g_pVW->put_Visible(OAFALSE);
        g_pVW->put_Owner(NULL);
    }

#ifdef REGISTER_FILTERGRAPH
    // Remove filter graph from the running object table   
    if (g_dwGraphRegister)
        RemoveGraphFromRot(g_dwGraphRegister);
#endif

    // Release DirectShow interfaces
	SAFE_RELEASE(g_pAnalogVideoDecoder);
	SAFE_RELEASE(g_pVSCC);
    SAFE_RELEASE(g_pMC);
    SAFE_RELEASE(g_pME);
    SAFE_RELEASE(g_pVW);
    SAFE_RELEASE(g_pGraph);
    SAFE_RELEASE(g_pCapture);
}

//------------------------------------------------------------------------------
// Function name	: FindCaptureDevice
// Description	    : Finds the first Video Capture Device in the system
// Return type		: HRESULT 
// Argument         : IBaseFilter ** ppSrcFilter
// Returns          : 
//
HRESULT FindCaptureDevice(IBaseFilter ** ppSrcFilter, int index)
{
    HRESULT hr;
    IBaseFilter * pSrc = NULL;
    CComPtr <IMoniker> pMoniker =NULL;
    ULONG cFetched;
   
    // Create the system device enumerator
    CComPtr <ICreateDevEnum> pDevEnum =NULL;

    hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
        IID_ICreateDevEnum, (void ** ) &pDevEnum);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create system enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // Create an enumerator for the video capture devices
    CComPtr <IEnumMoniker> pClassEnum = NULL;

    hr = pDevEnum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create class enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // If there are no enumerators for the requested type, then 
    // CreateClassEnumerator will succeed, but pClassEnum will be NULL.
    if (pClassEnum == NULL)
    {
        MessageBox(ghApp,TEXT("No video capture device was detected.\r\n\r\n")
                   TEXT("This sample requires a video capture device, such as a USB WebCam,\r\n")
                   TEXT("to be installed and working properly.  The sample will now close."),
                   TEXT("No Video Capture Hardware"), MB_OK | MB_ICONINFORMATION);
        return E_FAIL;
    }

    // Use the index video capture device on the device list.
	if(index)
		pClassEnum->Skip(index);

    // Note that if the Next() call succeeds but there are no monikers,
    // it will return S_FALSE (which is not a failure).  Therefore, we
    // check that the return code is S_OK instead of using SUCCEEDED() macro.
    if (S_OK == (pClassEnum->Next (1, &pMoniker, &cFetched)))
    {
        // Bind Moniker to a filter object
        hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**)&pSrc);
        if (FAILED(hr))
        {
            Msg(TEXT("Couldn't bind moniker to filter object!  hr=0x%x"), hr);
            return hr;
        }
    }
    else
    {
        Msg(TEXT("Unable to access video capture device!"));   
        return E_FAIL;
    }

    // Copy the found filter pointer to the output parameter.
    // Do NOT Release() the reference, since it will still be used
    // by the calling function.
    *ppSrcFilter = pSrc;

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: FindOverlayMixer
// Description	    : Finds the overlay mixer, allowing us the option of ensuring
//					  that we overlay not preview.
// Return type		: HRESULT 
// Argument         : IBaseFilter ** ppOverlay
// Returns          : 
//
HRESULT FindOverlayMixer(IBaseFilter ** ppOverlay)
{
    HRESULT hr;
    IBaseFilter * pOvr = NULL;
    CComPtr <IMoniker> pMoniker =NULL;
    ULONG cFetched;
   
    // Create the system device enumerator
    CComPtr <ICreateDevEnum> pDevEnum =NULL;

    hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, IID_ICreateDevEnum, (void ** ) &pDevEnum);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create system enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // Create an enumerator for the video capture devices
    CComPtr <IEnumMoniker> pClassEnum = NULL;

    hr = pDevEnum->CreateClassEnumerator (CLSID_LegacyAmFilterCategory, &pClassEnum, 0);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create class enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // If there are no enumerators for the requested type, then 
    // CreateClassEnumerator will succeed, but pClassEnum will be NULL.
    if (pClassEnum == NULL)
    {
		Msg(TEXT("pClassEnum == NULL"));
        return E_FAIL;
    }

	while(pClassEnum->Next(1, &pMoniker, &cFetched) == S_OK)
	{
		IPropertyBag *pPBag;
		wachTxtFriendlyName[0] = 0;
		hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPBag);
		if(SUCCEEDED(hr))
		{
			VARIANT var;
			var.vt = VT_BSTR;
			hr = pPBag->Read(L"FriendlyName", &var, NULL);
			if(hr==NOERROR) {
				lstrcpyW(wachTxtFriendlyName, var.bstrVal);
				SysFreeString(var.bstrVal);
				if(!lstrcmpW(L"Overlay Mixer", wachTxtFriendlyName))
					hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**)&pOvr);
			}
			pPBag->Release();
		}
	}

    *ppOverlay = pOvr;

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: BuildGraph
// Description	    : Builds the direct show filter graph that this app uses.
//					  Relies on some globals to make decisions when building
//					  graph.
// Return type		: HRESULT 
// Returns          :
//
HRESULT BuildGraph(int index)
{
    HRESULT hr;

    // Get DirectShow interfaces
    hr = GetInterfaces();
    if (FAILED(hr))
    {
        Msg("Failed to get video interfaces!  hr=0x%x", hr);
        return hr;
    }

    // Attach the filter graph to the capture graph
    hr = g_pCapture->SetFiltergraph(g_pGraph);
    if (FAILED(hr))
    {
        Msg("Failed to set capture filter graph!  hr=0x%x", hr);
        return hr;
    }

    // Use the system device enumerator and class enumerator to find
    // a video capture/preview device.
    hr = FindCaptureDevice(&g_pGRABBER, index);
    if (FAILED(hr))
    {
        // Don't display a message because FindCaptureDevice will handle it
        return hr;
    }
   
    // Add Capture filter to our graph.
    hr = g_pGraph->AddFilter(g_pGRABBER, L"Video Capture");
    if (FAILED(hr))
    {
        Msg("Couldn't add capture filter to graph!  hr=0x%x", hr);
		SAFE_RELEASE(g_pGRABBER);
        return hr;
    }

	// Get pointers to useful interfaces
	hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, g_pGRABBER, IID_IAMStreamConfig, (void **)&g_pVSCC);

	if(FAILED(hr))
	{
		Msg("Couldn't get IAMStreamConfig Interface (Capture)\n");
		SAFE_RELEASE(g_pGRABBER);
		return hr;
	}
	
	hr = g_pGRABBER->QueryInterface(IID_IAMAnalogVideoDecoder, (void **)&g_pAnalogVideoDecoder);
	if(FAILED(hr))
	{
		Msg("Couldn't get IAMAnalogVideoDecoder\n");
		SAFE_RELEASE(g_pGRABBER);
		return hr;
	}

	// default to PAL.
	g_pAnalogVideoDecoder->put_TVFormat(AnalogVideo_PAL_I);

	g_bGraphBuilt = TRUE;

	return S_OK;
}

HRESULT DestroyGraph()
{
	if(g_bGraphBuilt)
	{
		
		NukeDownstream(g_pGRABBER);
		g_pGraph->RemoveFilter(g_pGRABBER);
		SAFE_RELEASE(g_pGRABBER);

		CloseInterfaces();		

		g_bGraphBuilt = FALSE;
	}

	return S_OK;
}

HRESULT SetPreviewSize(int width, int height)
{
	HRESULT hr;

	// Set the video size on the capture pin to our desired size and format.
	if(g_pVSCC)
	{
		AM_MEDIA_TYPE *pmt;
		hr = g_pVSCC->GetFormat(&pmt);
		if(FAILED(hr))
		{
			Msg("Couldn't read current media format\n");
			return hr;
		}
		else
		{
			// need to check that we have the right format type.
			if(pmt->formattype == FORMAT_VideoInfo)
			{
				long biWidth, biHeight;
				DWORD biSizeImage;
				VIDEOINFOHEADER *pvih = (VIDEOINFOHEADER *)pmt->pbFormat;
				biWidth = width;
				biHeight = height;
				biSizeImage = width * height * 2;

				// If we are not using the overlay we will have to change
				// away from the default YUY2 data offered by the driver as
				// well as changing the size. If we were just to change the 
				// size at this point we would change the resolution offered in
				// YUY2 mode but as soon as we render the stream direct show 
				// would automatically switch us to RGB555 back at the default
				// resolution.
				/*
					// We need to change the color format from the default yuy2 to rgb16
					Msg("Setting MEDIA format to RGB555");
					pmt->subtype = MEDIASUBTYPE_RGB555;
					pvih->bmiHeader.biCompression = 0L;
				*/

				pvih->bmiHeader.biWidth = biWidth;
				pvih->bmiHeader.biHeight = biHeight;
				pvih->bmiHeader.biSizeImage = biSizeImage;

				hr = g_pVSCC->SetFormat(pmt);
				if(FAILED(hr))
				{
					Msg("Couldn't set media format - hr = 0x%08x\n", hr);
					return hr;
				}

			}
			DeleteMediaType(pmt);
		}
	}
	
    // Render the capture pin on the video capture filter
	hr = g_pCapture->RenderStream(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, g_pGRABBER, NULL, NULL);
    if (FAILED(hr))
    {
        Msg("Couldn't render capture stream. The device may already be in use.\r\n\r\nhr=0x%x", hr);
		g_pGRABBER->Release();
        return hr;
    }

	g_bGraphBuilt = TRUE;

    return S_OK;
}

//------------------------------------------------------------------------------
// Function name	: NukeDownstream
// Description	    : Tear down everything downstream of a given filter
// Return type		: void 
// Argument         : IBaseFilter *pf
// Returns          : 
//
void NukeDownstream(IBaseFilter *pf) 
{
    IPin *pP, *pTo;
    ULONG u;
    IEnumPins *pins = NULL;
    PIN_INFO pininfo;
    HRESULT hr = pf->EnumPins(&pins);
    pins->Reset();

    while(hr == NOERROR) 
	{
        hr = pins->Next(1, &pP, &u);
        if(hr == S_OK && pP) 
		{
            pP->ConnectedTo(&pTo);
            if(pTo) 
			{
                hr = pTo->QueryPinInfo(&pininfo);
                if(hr == NOERROR) 
				{
                    if(pininfo.dir == PINDIR_INPUT) 
					{
                        NukeDownstream(pininfo.pFilter);
                        g_pGraph->Disconnect(pTo);
                        g_pGraph->Disconnect(pP);
                        g_pGraph->RemoveFilter(pininfo.pFilter);
                    }
                    pininfo.pFilter->Release();
                }
                pTo->Release();
            }
            pP->Release();
        }
    }
    if(pins)
        pins->Release();
}

//------------------------------------------------------------------------------
// Function name	: NukeDownstream
// Description	    : Tear down everything downstream of a given filter
// Return type		: void 
// Argument         : IBaseFilter *pf
// Returns          : 
//
void DisconnectDownstream(IBaseFilter *pf) 
{
    IPin *pP, *pTo;
    ULONG u;
    IEnumPins *pins = NULL;
    PIN_INFO pininfo;
    HRESULT hr = pf->EnumPins(&pins);
    pins->Reset();

    while(hr == NOERROR) 
	{
        hr = pins->Next(1, &pP, &u);
        if(hr == S_OK && pP) 
		{
            pP->ConnectedTo(&pTo);
            if(pTo) 
			{
                hr = pTo->QueryPinInfo(&pininfo);
                if(hr == NOERROR) 
				{
                    if(pininfo.dir == PINDIR_INPUT) 
					{
                        DisconnectDownstream(pininfo.pFilter);
                        g_pGraph->Disconnect(pTo);
                        g_pGraph->Disconnect(pP);
                        //g_pGraph->RemoveFilter(pininfo.pFilter);
                    }
                    //pininfo.pFilter->Release();
                }
                pTo->Release();
            }
            pP->Release();
        }
    }
    if(pins)
        pins->Release();
}

//------------------------------------------------------------------------------
// Function name	: Capture
// Description	    : Start the graph.
// Return type		: HRESULT 
// Returns          : 
//
HRESULT Capture()
{
    HRESULT hr;

    // Set video window style and position
    hr = SetupVideoWindow();
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't initialize video window!  hr=0x%x"), hr);
        return hr;
    }

	// Add our graph to the running object table, which will allow
    // the GraphEdit application to "spy" on our graph
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(g_pGraph, &g_dwGraphRegister);
    if (FAILED(hr))
    {
        Msg(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr);
        g_dwGraphRegister = 0;
    }
#endif

    // Start previewing video data
    hr = g_pMC->Run();
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't run the graph!  hr=0x%x"), hr);
        return hr;
    }

    // Remember current state
    g_psCurrent = Running;
        
    return S_OK;
}


///////////////////////////////////////////////////////////////////////////////
/// SetupVideoWindow
/// 
/// Sets up the application window so that ic can contain our capture.
/// 
/// \param
///     void
/// 
/// \return
///     HRESULT 
///
/// \remark
///
///
HRESULT SetupVideoWindow(void)
{
    HRESULT hr = S_OK;

	if(g_pVW)
	{
		// Set the video window to be a child of the main window
		hr = g_pVW->put_Owner((OAHWND)ghApp);
		if (FAILED(hr))
			return hr;
    
		// Set video window style
		hr = g_pVW->put_WindowStyle(WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS );
		if (FAILED(hr))
			return hr;

		RECT rc;
		hr = GetClientRect(ghApp, &rc);
		long vidwidth, vidheight;
		hr = g_pVW->get_Width(&vidwidth);
		hr = g_pVW->get_Height(&vidheight);

		long diffx, diffy;
		diffx = (rc.right - rc.left) - vidwidth;
		diffy = (rc.bottom - rc.top) - vidheight;

		SetWindowPos(ghApp, HWND_TOP, 0, 0, (rc.right-rc.left)-diffx, (rc.bottom-rc.top)-diffy, SWP_NOMOVE|SWP_NOZORDER);

		// Use helper function to position video window in client rect of main application window
		ResizeVideoWindow();

		// Make the video window visible, now that it is properly positioned

		hr = g_pVW->put_Visible(OATRUE);
		if (FAILED(hr))
			return hr;
	}

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: ResizeVideoWindow
// Description	    : Correctly sizes our window.
// Return type		: void 
// Argument         : void
// Returns          : 
//
void ResizeVideoWindow(void)
{
    RECT rc;

    // Make the preview video fill our window
    GetClientRect(ghApp, &rc);

    // Resize the video preview window to match owner window size
    if (g_pVW)
        g_pVW->SetWindowPosition(0, 0, rc.right, rc.bottom);
}

//------------------------------------------------------------------------------
// Function name	: ChangePreviewState
// Description	    : Change the current running state of the graph
// Return type		: HRESULT 
// Argument         : int nShow
// Returns          : 
//
HRESULT ChangePreviewState(int nShow)
{
    HRESULT hr=S_OK;
    
    // If the media control interface isn't ready, don't call it
    if (!g_pMC)
        return S_OK;
    
    if (nShow)
    {
        if (g_psCurrent != Running)
        {
            // Start previewing video data
            hr = g_pMC->Run();
            g_psCurrent = Running;
        }
    }
    else
    {
        // Stop previewing video data
        hr = g_pMC->StopWhenReady();
        g_psCurrent = Stopped;
    }

    return hr;
}

#ifdef REGISTER_FILTERGRAPH

//------------------------------------------------------------------------------
// Function name	: AddGraphToRot
// Description	    : Utility function. Allows graph to be connected to using
//					  graphEdit from MS.
// Return type		: HRESULT 
// Argument         : IUnknown *pUnkGraph
// Argument         : DWORD *pdwRegister
// Returns          : 
//
HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
    IMoniker * pMoniker;
    IRunningObjectTable *pROT;
    WCHAR wsz[128];
    HRESULT hr;

    if (FAILED(GetRunningObjectTable(0, &pROT))) 
	{
        return E_FAIL;
    }

    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());

    hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) 
	{
        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();
    return hr;
}

//------------------------------------------------------------------------------
// Function name	: RemoveGraphFromRot
// Description	    : Utility function. Allows graph to be connected to using
//					  graphEdit from MS.
// Return type		: void 
// Argument         : DWORD pdwRegister
// Returns          : 
//
void RemoveGraphFromRot(DWORD pdwRegister)
{
    IRunningObjectTable *pROT;

    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) 
	{
        pROT->Revoke(pdwRegister);
        pROT->Release();
    }
}

#endif

//------------------------------------------------------------------------------
// Function name	: Msg
// Description	    : Simple message dump function
// Return type		: void 
// Argument         : TCHAR *szFormat
// Argument         : ...
// Returns          : 
//
void Msg(TCHAR *szFormat, ...)
{
    TCHAR szBuffer[512];

    va_list pArgs;
    va_start(pArgs, szFormat);
    _vstprintf_s(szBuffer, 512, szFormat, pArgs);
    va_end(pArgs);

    MessageBox(NULL, szBuffer, TEXT("PlayCap Message"), MB_OK | MB_ICONERROR);
}

//------------------------------------------------------------------------------
// Function name	: ToggleFullScreen
// Description	    : Toggles fullscreen display of the captured video
// Return type		: HRESULT 
// Argument         : void
// Returns          :
//
HRESULT ToggleFullScreen(void)
{
    HRESULT hr=S_OK;
    LONG lMode;
    static HWND hDrain=0;

    if (!g_pVW)
        return S_OK;

    // Read current state
    hr = g_pVW->get_FullScreenMode(&lMode);
	if(FAILED(hr))
	{
		Msg("get_FullScreenMode failed");
		return hr;
	}

    if (lMode == OAFALSE)
    {
        // Save current message drain
        hr = g_pVW->get_MessageDrain((OAHWND *) &hDrain);
		if(FAILED(hr))
			Msg("get_MessageDrain - hr = 0x%08x", hr);

        // Set message drain to application main window
        hr = g_pVW->put_MessageDrain((OAHWND) ghApp);
		if(FAILED(hr))
			Msg("put_MessageDrain - hr = 0x%08x", hr);

        // Switch to full-screen mode
        lMode = OATRUE;
        hr = g_pVW->put_FullScreenMode(lMode);
		if(FAILED(hr))
		{
			Msg("put_FullScreenMode failed");
			return hr;
		}

        g_bFullscreen = TRUE;
    }
    else
    {
        // Switch back to windowed mode
        lMode = OAFALSE;
        hr = g_pVW->put_FullScreenMode(lMode);
		if(FAILED(hr))
		{
			Msg("put_FullScreenMode failed");
			return hr;
		}

        // Undo change of message drain
        hr = g_pVW->put_MessageDrain((OAHWND) hDrain);
		if(FAILED(hr))
			Msg("put_MessageDrain - hr = 0x%08x", hr);

        // Reset video window
        hr = g_pVW->SetWindowForeground(-1);
		if(FAILED(hr))
			Msg("SetWindowForeground Failed - hr = 0x%08x", hr);

        // Reclaim keyboard focus for player application
        UpdateWindow(ghApp);
        SetForegroundWindow(ghApp);
        SetFocus(ghApp);
        g_bFullscreen = FALSE;
    }

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: DoDialog
// Description	    : brings up a dialog box
// Return type		: int 
// Argument         : HWND hwndParent
// Argument         : int DialogID
// Argument         : DLGPROC fnDialog
// Argument         : long lParam
// Returns          : 
//
int DoDialog(HWND hwndParent, int DialogID, DLGPROC fnDialog, long lParam) 
{
    DLGPROC fn;
    int result;

    fn = (DLGPROC)MakeProcInstance(fnDialog, ghInst);
    result = (int) DialogBoxParam(ghInst, MAKEINTRESOURCE(DialogID), hwndParent, fn, lParam);
    FreeProcInstance(fn);

    return result;
}

//------------------------------------------------------------------------------
// Function name	: HandleGraphEvent
// Description	    : 
// Return type		: HRESULT 
// Argument         : void
// Returns          : 
//
HRESULT HandleGraphEvent(void)
{
    LONG evCode, evParam1, evParam2;
    HRESULT hr=S_OK;

    while(SUCCEEDED(g_pME->GetEvent(&evCode, (LONG_PTR *) &evParam1, (LONG_PTR *) &evParam2, 0)))
    {
        // Free event parameters to prevent memory leaks associated with
        // event parameter data.  While this application is not interested
        // in the received events, applications should always process them.
        hr = g_pME->FreeEventParams(evCode, evParam1, evParam2);
        
        // Insert event processing code here, if desired
    }

    return hr;
}

//------------------------------------------------------------------------------
// Function name	: WndMainProc 
// Description	    : Main Window Proc
// Return type		: LRESULT CALLBACK 
// Argument         : HWND hwnd
// Argument         : UINT message
// Argument         : WPARAM wParam
// Argument         : LPARAM lParam
// Returns          : 
//
LRESULT CALLBACK WndMainProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HRESULT hr;
	int index;
	long lDecoder=0;

    switch (message)
    {
        case WM_GRAPHNOTIFY:
            HandleGraphEvent();
            break;

        case WM_KEYDOWN:

            switch(toupper((int) wParam))
            {
				// This toggles the input standard between PAL and NTSC.
				case 'I':
					g_pMC->Stop();
					DisconnectDownstream(g_pGRABBER);
					if(g_bPal)
					{
						g_pAnalogVideoDecoder->put_TVFormat(AnalogVideo_NTSC_M);
						g_bPal = FALSE;
					}
					else
					{
						g_pAnalogVideoDecoder->put_TVFormat(AnalogVideo_PAL_I);
						g_bPal = TRUE;
					}
					hr = SetPreviewSize(capsize[capsize_idx][g_bPal].width, capsize[capsize_idx][g_bPal].height);
					hr = Capture();
					ShowWindow(ghApp, SW_SHOWNORMAL);
					UpdateWindow(ghApp);
					SetForegroundWindow(ghApp);
					SetFocus(ghApp);
					g_bFullscreen = FALSE;
					break;

				// Display the property pages.
				case 'P':
					ISpecifyPropertyPages *pProp;
					hr = g_pGRABBER->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
					if (SUCCEEDED(hr)) 
					{
						// Get the filter's name and IUnknown pointer.
						FILTER_INFO FilterInfo;
						hr = g_pGRABBER->QueryFilterInfo(&FilterInfo); 
						IUnknown *pFilterUnk;
						g_pGRABBER->QueryInterface(IID_IUnknown, (void **)&pFilterUnk);

						// Show the page. 
						CAUUID caGUID;
						pProp->GetPages(&caGUID);
						pProp->Release();
						OleCreatePropertyFrame(
							ghApp,                   // Parent window
							0, 0,                   // Reserved
							FilterInfo.achName,     // Caption for the dialog box
							1,                      // Number of objects (just the filter)
							&pFilterUnk,            // Array of object pointers. 
							caGUID.cElems,          // Number of property pages
							caGUID.pElems,          // Array of property page CLSIDs
							0,                      // Locale identifier
							0, NULL                 // Reserved
						);

						// Clean up.
						pFilterUnk->Release();
						FilterInfo.pGraph->Release(); 
						CoTaskMemFree(caGUID.pElems);
					}
					break;

				case 'F':
                case VK_RETURN:
				case VK_SPACE:
                    ToggleFullScreen();
                    break;

				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
					index = wParam-'1';
					if(index!=g_Index)
					{
						g_Index=index;
						DestroyGraph();
						StartGraph(g_Index);
					}
					break;

				case 'S':
					g_pMC->Stop();
					break;

				case 'R':
					g_pMC->Run();
					break;

                case VK_ESCAPE:
                    if(g_bFullscreen)
                        ToggleFullScreen();
                    break;

				case VK_PRIOR:
					if(capsize_idx == (CAPTURE_SIZE_ENTRIES-1) )
					{
						// don't increment any further
					}
					else
					{
						g_pMC->Stop();
						capsize_idx++;
						DisconnectDownstream(g_pGRABBER);
						//hr = SetPreviewSize(capsize[capsize_idx][g_bPal].width, capsize[capsize_idx][g_bPal].height);
						g_VidWidth = capsize[capsize_idx][g_bPal].width;
						g_VidHeight = capsize[capsize_idx][g_bPal].height;
						hr = SetPreviewSize(g_VidWidth, g_VidHeight);
						hr = Capture();
						ShowWindow(ghApp, SW_SHOWNORMAL);
						UpdateWindow(ghApp);
						SetForegroundWindow(ghApp);
						SetFocus(ghApp);
						g_bFullscreen = FALSE;
						char szbuffer[80];
						sprintf_s(szbuffer, 80, "%dx%d", capsize[capsize_idx][g_bPal].width, capsize[capsize_idx][g_bPal].height);
						SetWindowText(ghApp, szbuffer);
					}
					break;

				case VK_NEXT:
					if(capsize_idx == 0 )
					{
						// don't decrement any further
					}
					else
					{
						g_pMC->Stop();
						capsize_idx--;
						DisconnectDownstream(g_pGRABBER);
						g_VidWidth = capsize[capsize_idx][g_bPal].width;
						g_VidHeight = capsize[capsize_idx][g_bPal].height;
						hr = SetPreviewSize(g_VidWidth, g_VidHeight);
						hr = Capture();
						ShowWindow(ghApp, SW_SHOWNORMAL);
						UpdateWindow(ghApp);
						SetForegroundWindow(ghApp);
						SetFocus(ghApp);
						g_bFullscreen = FALSE;
						char szbuffer[80];
						sprintf_s(szbuffer, 80, "%dx%d", capsize[capsize_idx][g_bPal].width, capsize[capsize_idx][g_bPal].height);
						SetWindowText(ghApp, szbuffer);
					}
					break;

				case VK_LEFT:
				case VK_RIGHT:
				case VK_UP:
				case VK_DOWN:
				{
					int sizechanged=0;
					if((wParam==VK_LEFT)&&(g_VidWidth>0))
					{
						g_VidWidth-=8;
						sizechanged=1;
					}
					else if((wParam==VK_RIGHT)&&(g_VidWidth<capsize[CAPTURE_SIZE_ENTRIES-1][g_bPal].width))
					{
						g_VidWidth+=8;
						sizechanged=1;
					}
					else if((wParam==VK_DOWN)&&(g_VidHeight>0))
					{
						g_VidHeight-=8;
						sizechanged=1;
					}
					else if((wParam==VK_UP)&&(g_VidHeight<capsize[CAPTURE_SIZE_ENTRIES-1][g_bPal].height))
					{
						g_VidHeight+=8;
						sizechanged=1;
					}
					if(sizechanged)
					{
						g_pMC->Stop();
						DisconnectDownstream(g_pGRABBER);
						hr = SetPreviewSize(g_VidWidth, g_VidHeight);
						hr = Capture();
						ShowWindow(ghApp, SW_SHOWNORMAL);
						UpdateWindow(ghApp);
						SetForegroundWindow(ghApp);
						SetFocus(ghApp);
						g_bFullscreen = FALSE;
						char szbuffer[80];
						sprintf_s(szbuffer, 80, "%dx%d", g_VidWidth, g_VidHeight);
						SetWindowText(ghApp, szbuffer);
					}
					break;
				}

				// Quits the application
				case 'Q':
		            PostMessage(hwnd,WM_CLOSE,0,0L);
					break;
			break;
            }
            break;

        case WM_CLOSE:            
            // Hide the main window while the graph is destroyed
            ShowWindow(ghApp, SW_HIDE);
            CloseInterfaces();  // Stop capturing and release interfaces
            break;

        case WM_DESTROY:
            PostQuitMessage(0);
            return 0;

		case WM_POWERBROADCAST:
			switch(wParam)
			{
                case PBT_APMQUERYSUSPEND:
                    // At this point we should prepare to go into a suspended mode. if we cannot return BROADCAST_QUERY_DENY
                    return TRUE;

				case PBT_APMSUSPEND:
					// No choice, we are suspending do what we can to get ourselves to the right state.
					break;

				case PBT_APMRESUMESUSPEND:
					// At this point we should restore our settings following the suspend.
					return TRUE;

				default:
					break;
			}
			break;
    }
    return DefWindowProc (hwnd , message, wParam, lParam);
}

// ----------------------------------------------------------------------------
// MAIN FUNCTION
// ----------------------------------------------------------------------------

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hInstP, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;

    WNDCLASS wc;

    // Initialize COM
    if(FAILED(CoInitialize(NULL)))
    {
        Msg(TEXT("CoInitialize Failed!\r\n"));   
        exit(1);
    } 

    // Register the window class
    ZeroMemory(&wc, sizeof wc);
    wc.lpfnWndProc   = WndMainProc;
    ghInst = wc.hInstance = hInstance;
    wc.lpszClassName = CLASSNAME;
    wc.lpszMenuName  = NULL;
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VIDPREVIEW));
    if(!RegisterClass(&wc))
    {
        Msg(TEXT("RegisterClass Failed! Error=0x%x\r\n"), GetLastError());
        CoUninitialize();
        exit(1);
    }

    // Create the main window.  The WS_CLIPCHILDREN style is required.
    ghApp = CreateWindow(CLASSNAME, 
					APPLICATIONNAME,
                    WS_OVERLAPPEDWINDOW | WS_CAPTION | WS_CLIPCHILDREN,
                    CW_USEDEFAULT, 
					CW_USEDEFAULT,
                    g_VidWidth, 
					g_VidHeight,
                    0, 
					0, 
					hInstance, 
					0);

    if(ghApp)
    {
		// Save menu handle for later use
		ghMenu = GetMenu(ghApp);

		ShowWindow(ghApp, nCmdShow);

		StartGraph(g_Index);

        // Main message loop
        while(GetMessage(&msg,NULL,0,0))
        {
			TranslateMessage(&msg) ;
			DispatchMessage(&msg) ;
        }
    }

    // Release COM
    CoUninitialize();

    return (int) msg.wParam;
}

void StartGraph(int index)
{
	HRESULT hr;

	if(!g_bGraphBuilt)
	{
		// Create DirectShow graph and start capturing video
		hr = BuildGraph(index);
		if (FAILED (hr))
		{
			CloseInterfaces();
			DestroyWindow(ghApp);
		}
		else
		{
			hr = SetPreviewSize(capsize[capsize_idx][g_bPal].width, capsize[capsize_idx][g_bPal].height);
			if(FAILED(hr))
			{
				CloseInterfaces();
				DestroyWindow(ghApp);
				return;
			}

			hr = Capture();
			if (FAILED (hr))
			{
				CloseInterfaces();
				DestroyWindow(ghApp);
				return;
			}
			
			ShowWindow(ghApp, SW_SHOWNORMAL);
			UpdateWindow(ghApp);
			SetForegroundWindow(ghApp);
			SetFocus(ghApp);
			g_bFullscreen = FALSE;
		}
	}
	else
		Msg("Graph Already Running");
}


